//GenericCControl.cpp - Holds all the properties etc. CControls may posess.

#include "wx\wxprec.h"

#ifndef WX_PRECOMP
#include "wx\wx.h"
#endif

#include "wx/image.h"

#include "GenericCControl.h"

//------------------------
GenericCControl::GenericCControl(const GenericCControl &control)
{
	unsigned long i;
	GenericProperty *tempprop;
	GenericImage *tempimage;

	strcpy(Type, control.Type);
	NumImages = control.NumImages;
	HasGUIImage = control.HasGUIImage;
	NumStacked = control.NumStacked;

	pos_x = control.pos_x;
	pos_y = control.pos_y;
	size_width = control.size_width;
	size_height = control.size_height;

	mouse_offset.x = 0;
	mouse_offset.y = 0;

	for(i=0;i<control.Properties.GetCount();i++)
	{
		tempprop = new GenericProperty;
		*tempprop = *control.Properties.Item(i);

		Properties.Add(tempprop);
	}

	for(i=0;i<control.Images.GetCount();i++)
	{
		tempimage = new GenericImage;
		*tempimage = *control.Images.Item(i);

		Images.Add(tempimage);
	}
}

//------------------------
GenericCControl::GenericCControl(const char *type)
{
	strcpy(Type, type);

	NumImages = 0;
	HasGUIImage = false;
	NumStacked = 0;

	pos_x = 0;
	pos_y = 0;
	size_width = 0;
	size_height = 0;
}

//------------------------
GenericCControl::~GenericCControl()
{
	unsigned long i;
	GenericProperty *temppoint;
	GenericImage *temppoint2;

	if(Properties.GetCount() > 0)
	{
		for(i=Properties.GetCount();i>0;i--)
		{
			temppoint = Properties.Item(i-1); //-1 because i is unsigned
			delete temppoint;
			Properties.RemoveAt(i-1);
		}
	}


	if(Images.GetCount() > 0)
	{
		for(i=Images.GetCount();i>0;i--)
		{
			temppoint2 = Images.Item(i-1);
			if(temppoint2->image)
				delete temppoint2->image;
			delete temppoint2;
			Images.RemoveAt(i-1);
		}
	}
}

//------------------------
GenericCControl GenericCControl::operator=(const GenericCControl &control)
{
	unsigned long i;
	GenericProperty *tempprop;
	GenericImage *tempimage;

	strcpy(Type, control.Type);
	NumImages = control.NumImages;
	HasGUIImage = control.HasGUIImage;
	NumStacked = control.NumStacked;

	pos_x = control.pos_x;
	pos_y = control.pos_y;
	size_width = control.size_width;
	size_height = control.size_height;

	for(i=0;i<control.Properties.GetCount();i++)
	{
		tempprop = new GenericProperty;
		*tempprop = *control.Properties.Item(i);

		Properties.Add(tempprop);
	}

	for(i=0;i<control.Images.GetCount();i++)
	{
		tempimage = new GenericImage;
		*tempimage = *control.Images.Item(i);

		Images.Add(tempimage);
	}

	return *this;
}

//------------------------
void GenericCControl::AddProperty(const char *name)
{
	GenericProperty *temppoint;

	temppoint = new GenericProperty;
	strcpy(temppoint->name, name);

	Properties.Add(temppoint);
}

//------------------------
void GenericCControl::SetProperty(const char *name, wxString val)
{
	unsigned long i;

	for(i=0;i<Properties.GetCount();i++)
	{
		if(!strcmp(Properties.Item(i)->name, name))
		{
			Properties.Item(i)->value = val;

			if(!strcmp(name, "pos_x"))
				val.ToLong(&pos_x);
			if(!strcmp(name, "pos_y"))
				val.ToLong(&pos_y);
			if(!strcmp(name, "size_width"))
				val.ToLong(&size_width);
			if(!strcmp(name, "size_height"))
				val.ToLong(&size_height);
			break;
		}
	}
}

//------------------------
wxString GenericCControl::GetProperty(const char *name)
{
	unsigned long i;

	for(i=0;i<Properties.GetCount();i++)
	{
		if(!strcmp(Properties.Item(i)->name, name))
		{
			return Properties.Item(i)->value;
		}
	}

	return wxString("");
}

//------------------------
char *GenericCControl::GetPropertyName(unsigned long i)
{
	if(i < Properties.GetCount())
		return Properties.Item(i)->name;
	else
		return 0;
}

//------------------------
void GenericCControl::SetCControlName(const char *type)
{
	strcpy(Type, type);
}

//------------------------
char *GenericCControl::GetCControlName()
{
	return Type;
}

//------------------------
void GenericCControl::AddImageProperty(const char *name)
{
	GenericImage *temppoint;

	temppoint = new GenericImage;
	strcpy(temppoint->name, name);
	temppoint->image = 0;

	Images.Add(temppoint);
	NumImages = Images.GetCount();
}

//------------------------
void GenericCControl::SetImagePath(wxString path, const char *name)
{
	unsigned long i;

	for(i=0;i<Images.GetCount();i++)
	{
		if(!strcmp(Images.Item(i)->name, name))
		{
			Images.Item(i)->path = path;
			break;
		}
	}
}

//------------------------
wxString GenericCControl::GetImagePath(const char *name)
{
	unsigned long i;

	for(i=0;i<Images.GetCount();i++)
	{
		if(!strcmp(Images.Item(i)->name, name))
		{
			return Images.Item(i)->path;
		}
	}
	return wxString("");
}

//------------------------
void GenericCControl::SetImageHandle(wxString handle, const char *name)
{
	unsigned long i;

	for(i=0;i<Images.GetCount();i++)
	{
		if(!strcmp(Images.Item(i)->name, name))
		{
			Images.Item(i)->handle = handle;
			break;
		}
	}
}

//------------------------
wxString GenericCControl::GetImageHandle(const char *name)
{
	unsigned long i;

	for(i=0;i<Images.GetCount();i++)
	{
		if(!strcmp(Images.Item(i)->name, name))
		{
			return Images.Item(i)->handle;
		}
	}
	return wxString("");
}

//------------------------
void GenericCControl::SetImage(wxImage *image, const char *name)
{
	unsigned long i;

	for(i=0;i<Images.GetCount();i++)
	{
		if(!strcmp(Images.Item(i)->name, name))
		{
			if(Images.Item(i)->image)
				delete Images.Item(i)->image;
			Images.Item(i)->image = image;
			break;
		}
	}
}

//------------------------
char *GenericCControl::GetImageName(unsigned long i)
{
	if(i < Images.GetCount())
		return Images.Item(i)->name;
	else
		return 0;
}

//------------------------
void GenericCControl::DrawImage(wxDC *dc)
{
	if(HasGUIImage)
	{
		wxBitmap Bitmap(Images.Item(0)->image);
		Bitmap.SetHeight(size_height);
		if((!GetProperty("transparent").Cmp(wxString("false")))||(!GetProperty("transparent").Cmp(wxString("FALSE")))||(!GetProperty("transparent").Cmp(wxString("0"))))
		{
			dc->SetPen(*wxWHITE_PEN);
			dc->SetBrush(*wxBLACK_BRUSH);
			dc->DrawRectangle(pos_x, pos_y, size_width, size_height);
			dc->DrawBitmap(Bitmap, pos_x, pos_y, TRUE);
		}
		else
			dc->DrawBitmap(Bitmap, pos_x, pos_y, TRUE);
	}
	else
	{
		dc->SetPen(*wxWHITE_PEN);
		dc->SetBrush(*wxBLACK_BRUSH);
		dc->DrawRectangle(pos_x, pos_y, size_width, size_height);
	}
}

//------------------------
void GenericCControl::SetMouseOffset(wxPoint point)
{
	mouse_offset.x = point.x - pos_x;
	mouse_offset.y = point.y - pos_y;
}

//------------------------
void GenericCControl::Move(wxPoint point)
{
	unsigned long i;

	pos_x = point.x - mouse_offset.x;
	pos_y = point.y - mouse_offset.y;

	for(i=0;i<Properties.GetCount();i++)
	{
		if(!strcmp(Properties.Item(i)->name, "pos_x"))
		{
			Properties.Item(i)->value.Printf("%d", pos_x);
		}
		if(!strcmp(Properties.Item(i)->name, "pos_y"))
		{
			Properties.Item(i)->value.Printf("%d", pos_y);
		}
	}
}

//------------------------
bool GenericCControl::IsInside(wxPoint point)
{
	wxRect temprect(pos_x, pos_y, size_width, size_height);

	return temprect.Inside(point);
}

//------------------------
void GenericCControl::MakeEmptyPropertiesZero()
{
	unsigned long i;

	for(i=0;i<Properties.GetCount();i++)
	{
		if(!Properties.Item(i)->value.Cmp(""))
		{
			Properties.Item(i)->value.Printf("0");
		}
	}

	for(i=0;i<Images.GetCount();i++)
	{
		if(!Images.Item(i)->handle.Cmp(""))
		{
			Images.Item(i)->handle.Printf("0");
		}
	}
}

//------------------------
void GenericCControl::SetPosition(long x, long y)
{
	unsigned long i;

	pos_x = x;
	pos_y = y;

	for(i=0;i<Properties.GetCount();i++)
	{
		if(!strcmp(Properties.Item(i)->name, "pos_x"))
		{
			Properties.Item(i)->value.Printf("%d", pos_x);
		}
		if(!strcmp(Properties.Item(i)->name, "pos_y"))
		{
			Properties.Item(i)->value.Printf("%d", pos_y);
		}
	}
}

//------------------------
void GenericCControl::SetSize(long width, long height)
{
	unsigned long i;

	size_width = width;
	size_height = height;

	for(i=0;i<Properties.GetCount();i++)
	{
		if(!strcmp(Properties.Item(i)->name, "size_width"))
		{
			Properties.Item(i)->value.Printf("%d", size_width);
		}
		if(!strcmp(Properties.Item(i)->name, "size_height"))
		{
			Properties.Item(i)->value.Printf("%d", size_height);
		}
	}
}