//GenericCControl.h - Holds all the properties etc. CControls may posess.

#ifndef GENERICCCONTROL_H_
#define GENERICCCONTROL_H_

//------------------------
typedef struct
{
	wxString value;
	char name[128];
} GenericProperty;

//------------------------
typedef struct
{
	wxString handle;
	char name[128];
	wxString path;
	wxImage *image;
} GenericImage;

WX_DEFINE_ARRAY(GenericProperty *, GenericPropertyArray);
WX_DEFINE_ARRAY(GenericImage *, GenericImageArray);

//------------------------
class GenericCControl
{
  public:
	GenericCControl(const GenericCControl &control);
	GenericCControl(const char *type);
	~GenericCControl();

	GenericCControl operator=(const GenericCControl &control);

	void AddProperty(const char *name);
	void SetProperty(const char *name, wxString val);
	wxString GetProperty(const char *name);
	unsigned long GetNumProperties() {return Properties.GetCount();};
	char *GetPropertyName(unsigned long i);

	void SetCControlName(const char *type);
	char *GetCControlName();

	//adds an image property to this class
	void AddImageProperty(const char *name);
	//sets the 'name'd image's path (used to eliminate duplicate image handles)
	void SetImagePath(wxString path, const char *name);
	//returns the 'name'd image's path (used to eliminate duplicate image handles)
	wxString GetImagePath(const char *name);
	//sets the 'name'd image's handle (e.g. hButton etc.)
	void SetImageHandle(wxString handle, const char *name);
	wxString GetImageHandle(const char *name);
	//sets the wxImage used to draw the control by VSTGUI Builder
	void SetImage(wxImage *image, const char *name);
	char *GetImageName(unsigned long i);


	void DrawImage(wxDC *dc);
	void SetMouseOffset(wxPoint point);
	void Move(wxPoint point);
	bool IsInside(wxPoint point);
	void MakeEmptyPropertiesZero();


	int GetNumImages() {return NumImages;};
	void SetNumImages(int val) {NumImages = val;};

	bool GetHasGUIImage() {return HasGUIImage;};
	void SetHasGUIImage(bool val) {HasGUIImage = val;};

	int GetNumStacked() {return NumStacked;};
	void SetNumStacked(int val) {NumStacked = val;};

	long GetPos_x() {return pos_x;};
	long GetPos_y() {return pos_x;};
	void SetPosition(long x, long y);
	long GetSize_width() {return size_width;};
	long GetSize_height() {return size_height;};
	void SetSize(long width, long height);
  protected:
	GenericPropertyArray Properties;
	GenericImageArray Images;

	char Type[128]; //e.g. CButton, CSlider etc.

	int NumImages;
	bool HasGUIImage;
	int NumStacked;

	long pos_x, pos_y;
	long size_width, size_height;

	wxPoint mouse_offset;
};

#endif