//VSTGUIBuilderFrame.cpp - png2ico frame class

#include "wx\wxprec.h"

#ifndef WX_PRECOMP
#include "wx\wx.h"
#endif

#include "VSTGUIBuilderFrame.h"
#include "AboutBox.h"

BEGIN_EVENT_TABLE(VSTGUIBuilderFrame, wxFrame)
	EVT_SIZE(VSTGUIBuilderFrame::OnSize)
	EVT_MENU(FILE_MENU_NEW, VSTGUIBuilderFrame::OnNew)
	EVT_MENU(FILE_MENU_EXIT, VSTGUIBuilderFrame::OnExit)
	EVT_MENU(FILE_MENU_CREATECODE, VSTGUIBuilderFrame::OnGenerateCode)
	EVT_MENU(INSERT_MENU_NEW_BACKGROUND, VSTGUIBuilderFrame::OnBackground)
	EVT_MENU(INSERT_MENU_CONTROL, VSTGUIBuilderFrame::OnInsertControl)
	EVT_MENU(HELP_MENU_ABOUT, VSTGUIBuilderFrame::OnAbout)
END_EVENT_TABLE()


VSTGUIBuilderFrame::VSTGUIBuilderFrame(const wxChar *title, int xpos, int ypos, int width, int height)
           : wxFrame((wxFrame *)NULL, -1, title, wxPoint(xpos, ypos), wxSize(width, height), wxDEFAULT_FRAME_STYLE)
{
	PropertiesPanel = 0;
	MainGUIPanel = 0;

	SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));

	//--Menu stuff--------
	wxMenu *FileMenu = new wxMenu();
	FileMenu->Append(FILE_MENU_NEW, wxString("&New"));
	FileMenu->Append(FILE_MENU_OPEN, wxString("&Open"));
	FileMenu->Append(FILE_MENU_SAVE, wxString("&Save"));
	FileMenu->AppendSeparator();
	FileMenu->Append(FILE_MENU_CREATECODE, wxString("Generate C++ Code"));
	FileMenu->AppendSeparator();
	FileMenu->Append(FILE_MENU_EXIT, wxString("E&xit"));

	//****For now****
	FileMenu->Enable(FILE_MENU_OPEN, FALSE);
	FileMenu->Enable(FILE_MENU_SAVE, FALSE);

	wxMenu *InsertMenu = new wxMenu();
	InsertMenu->Append(INSERT_MENU_NEW_BACKGROUND, wxString("New Background"));
	InsertMenu->Append(INSERT_MENU_CONTROL, wxString("Control"));

	wxMenu *HelpMenu = new wxMenu();
	HelpMenu->Append(HELP_MENU_ABOUT, wxString("About"));

	Menu = new wxMenuBar();
	Menu->Append(FileMenu, wxString("File"));
	Menu->Append(InsertMenu, wxString("Insert"));
	Menu->Append(HelpMenu, wxString("Help"));

	SetMenuBar(Menu);

	//--Panel stuff--------
	PropertiesPanel = new VSTGUIPropertiesPanel(this, -1, wxPoint(0, 0), wxSize(200, GetClientSize().GetHeight()), wxTAB_TRAVERSAL|wxCLIP_CHILDREN);
	MainGUIPanel = new VSTMainGUIPanel(this, -1, wxPoint(200, 0), wxSize((GetClientSize().GetWidth()-200), GetClientSize().GetHeight()), wxSUNKEN_BORDER|wxCLIP_CHILDREN, "panel", PropertiesPanel);


	wxIcon icon(wxICON(VSTGUIB_ICON));
	SetIcon(icon);
}

VSTGUIBuilderFrame::~VSTGUIBuilderFrame()
{
}

void VSTGUIBuilderFrame::OnSize(wxSizeEvent& event)
{
	if(PropertiesPanel)
		PropertiesPanel->SetSize(0, 0, 200, GetClientSize().GetHeight());
	if(MainGUIPanel)
		MainGUIPanel->SetSize(200, 0, (GetClientSize().GetWidth()-200), GetClientSize().GetHeight());
}

void VSTGUIBuilderFrame::OnBackground(wxCommandEvent& event)
{
	wxFileDialog dlg(this, wxString("Choose an image file"), wxString(""), wxString(""), wxString("png files (*.png)|*.png|bmp files (*.bmp)|*.bmp"), wxOPEN);

	if(dlg.ShowModal() == wxID_OK)
	{
		MainGUIPanel->LoadBackgroundImage(dlg.GetPath());
	}
}

void VSTGUIBuilderFrame::OnGenerateCode(wxCommandEvent& event)
{
	wxFileDialog dlg(this, wxString("Editor Name? (no extension)"), wxString(""), wxString(""), wxString("any file (*.*)|*.*"), wxSAVE);

	if(dlg.ShowModal() == wxID_OK)
	{
		MainGUIPanel->GenerateCode(dlg.GetFilename());
	}
}

void VSTGUIBuilderFrame::Refresh(bool eraseBackground, const wxRect* rect)
{
	wxWindow::Refresh(eraseBackground, rect);

	//does it not do this already?
	MainGUIPanel->Refresh();
	PropertiesPanel->Refresh();
}

void VSTGUIBuilderFrame::OnAbout(wxCommandEvent& event)
{
	AboutBox about(this, -1);
	about.ShowModal();
}