//VSTGUIPropertiesPanel.cpp - Panel to display the properties of the selected CControl

#include "wx\wxprec.h"

#ifndef WX_PRECOMP
#include "wx\wx.h"
#endif

#include "wx/image.h"
#include "VSTGUIPropertiesPanel.h"

BEGIN_EVENT_TABLE(VSTGUIPropertiesPanel, wxWindow)
    EVT_GRID_CELL_CHANGE(VSTGUIPropertiesPanel::OnGrid)
	//EVT_ERASE_BACKGROUND(VSTGUIPropertiesPanel::OnErase)
	EVT_COMBOBOX(IMAGE_SELECTED, VSTGUIPropertiesPanel::OnImageSelect)
	EVT_BUTTON(LOAD_IMAGE, VSTGUIPropertiesPanel::OnImageLoad)
	EVT_TEXT_ENTER(IMAGE_PATH, VSTGUIPropertiesPanel::OnImagePath)
END_EVENT_TABLE()

//------------------------
VSTGUIPropertiesPanel::VSTGUIPropertiesPanel(wxWindow* parent,
						  wxWindowID id,
						  const wxPoint& pos,
						  const wxSize& size,
						  long style,
						  const wxString& name):
wxPanel(parent, id, pos, size, style, name)
{
	wxFont tempfont(8, wxDEFAULT, wxNORMAL, wxBOLD);

	PropertiesLabel = new wxStaticText(this, -1, wxString("CControl Properties :"), wxPoint(4, 4));
	PropertiesLabel->SetFont(tempfont);

	ImageLabel = new wxStaticText(this, -1, wxString("Image :"), wxPoint(4, 20));
	ImageNum = new wxComboBox(this, IMAGE_SELECTED, wxString(""), wxPoint(2, 36), wxSize(166, 20), 0, 0, wxCB_READONLY);
	ImagePath = new wxTextCtrl(this, IMAGE_PATH, wxString(""), wxPoint(2, 60), wxSize(166, 20));
	ImageLoad = new wxButton(this, LOAD_IMAGE, wxString("..."), wxPoint(176, 60), wxSize(20, 20));

	ActualProperties = new wxGrid(this, -1, wxPoint(2, 82), wxSize(196, 360));

	ActualProperties->CreateGrid(17, 2);
	ActualProperties->SetColSize(0, 80);
    ActualProperties->SetColSize(1, 116);
	ActualProperties->SetLabelSize(wxVERTICAL, 0);
	ActualProperties->SetLabelSize(wxHORIZONTAL, 20);
	ActualProperties->SetLabelValue(wxHORIZONTAL, wxString("Property"), 0);
	ActualProperties->SetLabelValue(wxHORIZONTAL, wxString("Value"), 1);
	for(unsigned char i=0;i<20;i++)
		ActualProperties->SetReadOnly(i, 0, TRUE);

	CurrentControl = 0;
}

//------------------------
void VSTGUIPropertiesPanel::OnGrid(wxGridEvent &event)
{
	int row;
	wxString tempstr;
	wxString tempstr2;
	long tempval = 1;
	long tempval2 = 1;

	row = event.GetRow();
	if(ActualProperties->GetCellValue(row, 0).Cmp(""))
	{
		if(CurrentControl != 0)
		{
			tempstr = ActualProperties->GetCellValue(row, 1);
			tempstr2 = ActualProperties->GetCellValue(row, 0);
			CurrentControl->SetProperty(tempstr2, tempstr);
			if(!ActualProperties->GetCellValue(row, 0).Cmp(wxString("num_subpixmaps")))
			{
				tempstr.ToLong(&tempval);
				CurrentControl->SetNumStacked(tempval);
				tempstr = CurrentControl->GetProperty("size_height");
				tempstr.ToLong(&tempval2);
				tempval = tempval2/tempval;
				tempstr.Printf("%d", tempval);
				CurrentControl->SetProperty(wxString("height_of_one_image"), tempstr);
				CurrentControl->SetProperty(wxString("size_height"), tempstr);
				UpdateProperties();
			}
			else if(!ActualProperties->GetCellValue(row, 0).Cmp(wxString("height_of_one_image")))
			{
				tempstr.ToLong(&tempval);
				CurrentControl->SetSize(CurrentControl->GetSize_width(), tempval);
				UpdateProperties();
			}

			GetParent()->Refresh();
		}
	}
}

//------------------------
void VSTGUIPropertiesPanel::OnImageSelect(wxCommandEvent &event)
{
	char tempch[128];
	wxString tempstr;

	tempstr = ImageNum->GetString(ImageNum->GetSelection());
	strcpy(tempch, tempstr);
	ImagePath->SetValue(CurrentControl->GetImagePath(tempch));
}

//------------------------
void VSTGUIPropertiesPanel::OnImageLoad(wxCommandEvent &event)
{
	char tempch[128];
	wxString tempstr;
	wxImage *tempimage;

	wxFileDialog dlg(this, wxString("Choose an image file"), wxString(""), wxString(""), wxString("png files (*.png)|*.png|bmp files (*.bmp)|*.bmp"), wxOPEN);

	if(dlg.ShowModal() == wxID_OK)
	{
		ImagePath->SetValue(dlg.GetPath());
		strcpy(tempch, CurrentControl->GetImageName(ImageNum->GetSelection()));
		CurrentControl->SetImagePath(dlg.GetPath(), tempch);
		tempimage = new wxImage(dlg.GetPath());
		CurrentControl->SetImage(tempimage, tempch);
		if(CurrentControl->GetHasGUIImage()&&(ImageNum->GetSelection() == 0))
		{
			tempstr.Printf("%d", tempimage->GetWidth());
			CurrentControl->SetProperty("size_width", tempstr);
			if(CurrentControl->GetNumStacked())
				tempstr.Printf("%d", (tempimage->GetHeight()/CurrentControl->GetNumStacked()));
			else
				tempstr.Printf("%d", tempimage->GetHeight());
			CurrentControl->SetProperty("size_height", tempstr);

			GetParent()->Refresh();
		}
	}
}

//------------------------
void VSTGUIPropertiesPanel::OnImagePath(wxCommandEvent &event)
{
	char tempch[128];
	wxString tempstr;

	if(!ImagePath->GetValue().Cmp(wxString("hBack")))
	{
		tempstr = ImageNum->GetString(ImageNum->GetSelection());
		strcpy(tempch, tempstr);
		CurrentControl->SetImagePath(wxString("hBack"), tempch);
		CurrentControl->SetImageHandle(wxString("hBack"), tempch);
	}
}

//------------------------
void VSTGUIPropertiesPanel::SetUpProperties(GenericCControl *control)
{
	unsigned long i;
	char tempch[128];
	wxString tempstr;

	CurrentControl = control;

	tempstr.Printf("%s Properties :", CurrentControl->GetCControlName());

	PropertiesLabel->SetLabel(tempstr);

	ImageNum->Clear();
	for(i=0;i<CurrentControl->GetNumImages();i++)
	{
		strcpy(tempch, CurrentControl->GetImageName(i));
		ImageNum->Append(tempch);
	}
	ImageNum->SetSelection(0);

	tempstr = ImageNum->GetString(0);
	strcpy(tempch, tempstr);
	ImagePath->SetValue(CurrentControl->GetImagePath(tempch));

	for(i=0;i<(unsigned)ActualProperties->GetNumberRows();i++)
	{
		ActualProperties->SetCellValue(i, 0, "");
		ActualProperties->SetCellValue(i, 1, "");
	}

	for(i=0;i<CurrentControl->GetNumProperties();i++)
	{
		strcpy(tempch, CurrentControl->GetPropertyName(i));
		ActualProperties->SetCellValue(i, 0, tempch);
		ActualProperties->SetCellValue(i, 1, CurrentControl->GetProperty(tempch));
	}

	Refresh();
}

//------------------------
void VSTGUIPropertiesPanel::UpdateProperties()
{
	unsigned long i;
	char tempch[128];

	if(!CurrentControl)
		return;

	for(i=0;i<CurrentControl->GetNumProperties();i++)
	{
		strcpy(tempch, CurrentControl->GetPropertyName(i));
		ActualProperties->SetCellValue(i, 1, CurrentControl->GetProperty(tempch));
	}
}

//------------------------
void VSTGUIPropertiesPanel::ClearProperties()
{
	unsigned long i;

	PropertiesLabel->SetLabel("CControl Properties :");

	ImageNum->Clear();
	ImagePath->SetValue(wxString(""));

	for(i=0;i<(unsigned)ActualProperties->GetNumberRows();i++)
	{
		ActualProperties->SetCellValue(i, 0, "");
		ActualProperties->SetCellValue(i, 1, "");
	}

	CurrentControl = 0;
}