//VSTMainGUIPanel.cpp - Panel to display the GUI stuff.

#include "wx\wxprec.h"

#ifndef WX_PRECOMP
#include "wx\wx.h"
#endif

#include "wx/image.h"
#include "wx/dcbuffer.h"
#include "wx/file.h"
#include "wx/arrimpl.cpp" // this is a magic incantation which must be done!

#include "tinyxml.h"

#include "VSTMainGUIPanel.h"
#include "InsertControlDialog.h"

WX_DEFINE_OBJARRAY(GenericCControlArray);

BEGIN_EVENT_TABLE(VSTMainGUIPanel, wxWindow)
    EVT_MOUSE_EVENTS(VSTMainGUIPanel::OnMouse)
    EVT_PAINT(VSTMainGUIPanel::OnPaint)
	EVT_ERASE_BACKGROUND(VSTMainGUIPanel::OnErase)
	EVT_KEY_UP(VSTMainGUIPanel::OnKey)
END_EVENT_TABLE()

//------------------------
VSTMainGUIPanel::VSTMainGUIPanel(wxWindow* parent,
						  wxWindowID id,
						  const wxPoint& pos,
						  const wxSize& size,
						  long style,
						  const wxString& name,
						  VSTGUIPropertiesPanel *proppanel):
wxPanel(parent, id, pos, size, style, name)
{
	wxInitAllImageHandlers();

	SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_APPWORKSPACE));

	PropertiesPanel = proppanel;

	ImageIsLoaded = false;
	Background = 0;
	Background = new wxImage();

	ControlIsSelected = false;
	SelectedControl = 0;

	TiXmlBase::SetCondenseWhiteSpace(false);

	SetUpControlsList();
}

//------------------------
VSTMainGUIPanel::~VSTMainGUIPanel()
{
	if(Background)
		delete Background;

	ControlsInUse.Empty();
	AvailableControls.Empty();
	ImageHandles.Empty();
}

//------------------------
void VSTMainGUIPanel::OnNew()
{
	ImageIsLoaded = false;
	if(Background)
		delete Background;
	Background = 0;
	Background = new wxImage();

	ControlsInUse.Empty();
	ImageHandles.Empty();

	PropertiesPanel->ClearProperties();
	Refresh();
}

//------------------------
void VSTMainGUIPanel::OnMouse(wxMouseEvent &event)
{
	unsigned long i;
	bool no_object = true;
	wxPoint temppoint = event.GetPosition();

	if(event.LeftDown())
	{
		for(i=0;i<ControlsInUse.GetCount();i++)
		{
			if(ControlsInUse.Item(i).IsInside(temppoint))
			{
				ControlsInUse.Item(i).SetMouseOffset(temppoint);
				SelectedControl = i;
				ControlIsSelected = true;
				PropertiesPanel->SetUpProperties(&ControlsInUse.Item(i));
				no_object = false;
				break;
			}
		}
		if(no_object)
			PropertiesPanel->ClearProperties();
	}
	else if(event.LeftIsDown()&&ControlIsSelected)
	{
		if(SelectedControl < ControlsInUse.GetCount())
		{
			ControlsInUse.Item(SelectedControl).Move(temppoint);
			PropertiesPanel->UpdateProperties();
			Refresh();
		}
	}
	else if(event.LeftUp())
	{
		PropertiesPanel->UpdateProperties();
		//PropertiesPanel->CleanUpProperties();
		ControlIsSelected = false;
	}
}

//------------------------
void VSTMainGUIPanel::OnPaint(wxPaintEvent &event)
{
	unsigned long i;

	wxBitmap MainBitmap(GetClientSize().GetWidth(), GetClientSize().GetHeight());
	wxBufferedPaintDC offscreendc(this, MainBitmap);

	wxBrush tempbrush(wxSystemSettings::GetColour(wxSYS_COLOUR_APPWORKSPACE), wxSOLID);
	wxPen temppen(wxSystemSettings::GetColour(wxSYS_COLOUR_APPWORKSPACE), 1, wxSOLID);
	offscreendc.SetPen(temppen);
	offscreendc.SetBrush(tempbrush);
	offscreendc.DrawRectangle(0, 0, GetClientSize().GetWidth(), GetClientSize().GetHeight());

	if(ImageIsLoaded)
	{
		wxBitmap BackgroundBitmap(Background);
		offscreendc.DrawBitmap(BackgroundBitmap, 0, 0, FALSE);
	}
	for(i=0;i<ControlsInUse.GetCount();i++)
	{
		ControlsInUse.Item(i).DrawImage(&offscreendc);
	}
}

//------------------------
void VSTMainGUIPanel::OnKey(wxKeyEvent &event)
{
	unsigned long i;
	GenericCControl *tempcontrol;

	if(event.GetKeyCode() == WXK_DELETE)
	{
		tempcontrol = PropertiesPanel->GetSelectedControl();
		if(tempcontrol)
		{
			for(i=0;i<ControlsInUse.GetCount();i++)
			{
				if(tempcontrol == &ControlsInUse.Item(i))
				{
					PropertiesPanel->ClearProperties();
					ControlsInUse.RemoveAt(i);
					Refresh();
					break;
				}
			}
		}
	}
}

//------------------------
void VSTMainGUIPanel::SetUpControlsList()
{
	wxString tempstr;
	GenericCControl *tempcontrol = 0;
	TiXmlNode *ac_parent;
	TiXmlNode *ac_child;
	TiXmlNode *ic_parent;
	TiXmlNode *ic_child;
	TiXmlNode *prop_parent;
	TiXmlNode *prop_child;
	TiXmlText *temptext;

	TiXmlDocument doc("DefaultControls.xml");
	if(!doc.LoadFile())
	{
		tempstr.Printf("Could not load config file 'DefaultControls.xml'. Error='%s'.\n", doc.ErrorDesc());
		wxMessageDialog tempdlg(this, tempstr, wxString("Please!  Not the glass figurine!"));
		tempdlg.ShowModal();
	}

	TiXmlHandle docHandle(&doc);

	//get the available CControl definitions
	ac_parent = docHandle.FirstChild("root").FirstChild("available_CControls").Node();
	ac_child = 0;
	if(ac_parent)
	{
		//--this loop goes through all the documented CControls----
		while(ac_child = ac_parent->IterateChildren(ac_child))
		{
			ic_parent = ac_child;
			ic_child = 0;
			if(ic_parent)
			{
				if(!ic_parent->ToElement())
					continue;
				tempcontrol = new GenericCControl(ic_parent->Value());

				//--this loop goes through the different types of data inside the control (settings, properties, images)----
				while(ic_child = ic_parent->IterateChildren(ic_child))
				{
					if(!ic_child->ToElement())
						continue;
					prop_parent = ic_child;
					prop_child = 0;
					if(prop_parent)
					{
						if(!strcmp(prop_parent->Value(), "settings"))
						{
							//--this loop goes through the settings of the control----
							while(prop_child = prop_parent->IterateChildren(prop_child))
							{
								if(!prop_child->ToElement())
									continue;
								temptext = prop_child->FirstChild()->ToText();
								if(!temptext)
									continue;
								if(!strcmp(prop_child->ToElement()->Value(), "NumImages"))
									tempcontrol->SetNumImages(atoi(temptext->Value()));
								if(!strcmp(prop_child->ToElement()->Value(), "NumStacked"))
									tempcontrol->SetNumStacked(atoi(temptext->Value()));
								if(!strcmp(prop_child->ToElement()->Value(), "HasGUIImage"))
								{
									if(!strcmp(temptext->Value(), "true"))
										tempcontrol->SetHasGUIImage(true);
									else
										tempcontrol->SetHasGUIImage(false);
								}
							}
						}
						if(!strcmp(prop_parent->Value(), "properties"))
						{
							//--this loop goes through the properties of the control----
							while(prop_child = prop_parent->IterateChildren(prop_child))
							{
								if(!prop_child->ToElement())
									continue;
								tempcontrol->AddProperty(prop_child->Value());
							}
						}
						if(!strcmp(prop_parent->Value(), "images"))
						{
							//--this loop goes through the properties of the control----
							while(prop_child = prop_parent->IterateChildren(prop_child))
							{
								if(!prop_child->ToElement())
									continue;
								tempcontrol->AddImageProperty(prop_child->Value());
							}
						}
					}
				}
				AvailableControls.Add(tempcontrol);
			}
		}
	}
}

//------------------------
void VSTMainGUIPanel::GenerateCode(wxString path, wxString name)
{
	unsigned long i, j;
	wxString tempstr2, tempstr3;
	wxString tempstr(path);
	tempstr += wxString(".h");
	wxFile header(tempstr, wxFile::write);

	WriteImageHandles();

	for(i=0;i<ControlsInUse.GetCount();i++)
		ControlsInUse.Item(i).MakeEmptyPropertiesZero();

	TiXmlDocument doc("DefaultControls.xml");
	if(!doc.LoadFile())
	{
		tempstr.Printf("Could not load config file 'DefaultControls.xml'. Error='%s'.\n", doc.ErrorDesc());
		wxMessageDialog tempdlg(this, tempstr, wxString("Look!  An elephant!"));
		tempdlg.ShowModal();
	}

	//TiXmlBase::SetCondenseWhiteSpace(false);

	TiXmlHandle docHandle(&doc);

//--Header first----------------

	//get the contents of the header code
	TiXmlNode *parent = docHandle.FirstChild("root").FirstChild("code_body").FirstChild("code").FirstChild("header").Node();
	TiXmlNode *child = 0;
	TiXmlText *temptxt = 0;
	if(parent)
	{
		while(child = parent->IterateChildren(child))
		{
			if(!strcmp(child->Value(), "name")) //replace <name/> with editor's name
			{
				header.Write(name);
			}
			else if(!strcmp(child->Value(), "controls_go_here")) //replace <controls_go_here/> with the controls declarations
			{
				for(i=0;i<ControlsInUse.GetCount();i++)
				{
					tempstr.Printf("    %s *%s;\n", ControlsInUse.Item(i).GetCControlName(), ControlsInUse.Item(i).GetProperty("name"));
					header.Write(tempstr);
				}
				header.Write(wxString("\n"));
			}
			else if(!strcmp(child->Value(), "bitmaps_go_here")) //replace <images_go_here/> with the CBitmaps declarations
			{
				for(i=0;i<ImageHandles.GetCount();i++)
				{
					tempstr.Printf("    CBitmap *%s;\n", ImageHandles.Item(i));
					header.Write(tempstr);
				}
			}
			else if(!strcmp(child->Value(), "br")) //replace <br/> with a line break
			{
				header.Write(wxString("\n"));
			}
			else if(!strcmp(child->Value(), "nbsp")) //replace <nbsp/> with a space
			{
				header.Write(wxString(" "));
			}
			else if(!strcmp(child->Value(), "tab")) //replace <tab/> with a tab
			{
				header.Write(wxString("    ")); //tab = 4 spaces
			}
			//Any other element refers to a code block which can be found in
			//the <available_CControls> section, to be substitued in for every
			//object.
			else if(child->Type() == 1) //for some reason, ELEMENT is picked up as an undeclared identifier?
			{
				WriteCodeBlock(&header, &doc, child->Value());
			}
			else //everything else is text, write it to the file
			{
				temptxt = child->ToText();
				if(temptxt)
				{
					tempstr = temptxt->Value();
					header.Write(tempstr);
				}
			}
		}
	}

//--Now the source----------------

	tempstr = path;
	tempstr += wxString(".cpp");
	wxFile source(tempstr, wxFile::write);

	//get the contents of the source code
	parent = docHandle.FirstChild("root").FirstChild("code_body").FirstChild("code").FirstChild("source").Node();
	child = 0;
	temptxt = 0;
	if(parent)
	{
		while(child = parent->IterateChildren(child))
		{
			if(!strcmp(child->Value(), "name")) //replace <name/> with editor's name
			{
				source.Write(name);
			}
			//replace <image_tags_go_here/> with image tags
			else if(!strcmp(child->Value(), "image_tags_go_here"))
			{
				for(i=0;i<ImageHandles.GetCount();i++)
				{
					tempstr2 = ImageHandles.Item(i);
					tempstr2.SetChar(0, 'k');
					tempstr.Printf("    %s,\n", tempstr2);
					source.Write(tempstr);
				}
			}
			//replace <controls_go_here_equal_0/> with control<name/> = 0;
			else if(!strcmp(child->Value(), "controls_go_here_equal_0"))
			{
				for(i=0;i<ControlsInUse.GetCount();i++)
				{
					tempstr.Printf("    %s = 0;\n", ControlsInUse.Item(i).GetProperty("name"));
					source.Write(tempstr);
				}
				source.Write(wxString("\n"));
			}
			else if(!strcmp(child->Value(), "images_go_here_equal_0"))
			{
				for(i=0;i<ImageHandles.GetCount();i++)
				{
					tempstr.Printf("    %s = 0;\n", ImageHandles.Item(i));
					source.Write(tempstr);
				}
				source.Write(wxString("\n"));
			}
			//****this bit will need to be changed if I want proper flexibility****
			//replace <bitmaps_constructors_go_here/> with the bitmaps' constructors
			else if(!strcmp(child->Value(), "bitmaps_constructors_go_here"))
			{
				for(i=0;i<ImageHandles.GetCount();i++)
				{
					tempstr3 = ImageHandles.Item(i);
					tempstr2 = ImageHandles.Item(i);
					tempstr2.SetChar(0, 'k');
					//tempstr.Printf("    if(!%s)\n", tempstr3);
					//source.Write(tempstr);
					tempstr.Printf("    %s = new CBitmap(%s);\n", tempstr3, tempstr2);
					source.Write(tempstr);
				}
				source.Write(wxString("\n"));
			}
			//****this bit will need to be changed if I want proper flexibility****
			//replace <bitmaps_constructors_go_here/> with the bitmaps' destructors
			else if(!strcmp(child->Value(), "bitmaps_destructors_go_here"))
			{
				for(i=0;i<ImageHandles.GetCount();i++)
				{
					tempstr3 = ImageHandles.Item(i);
					tempstr.Printf("    if(%s)\n", tempstr3);
					source.Write(tempstr);
					tempstr.Printf("        %s->forget();\n", tempstr3);
					source.Write(tempstr);
					//tempstr.Printf("    %s = 0;\n", tempstr3);
					//source.Write(tempstr);
				}
				source.Write(wxString("\n"));
			}
			else if(!strcmp(child->Value(), "br")) //replace <br/> with a line break
			{
				source.Write(wxString("\n"));
			}
			else if(!strcmp(child->Value(), "nbsp")) //replace <nbsp/> with a space
			{
				source.Write(wxString(" "));
			}
			else if(!strcmp(child->Value(), "tab")) //replace <tab/> with a tab
			{
				source.Write(wxString("    ")); //tab = 4 spaces
			}
			//Any other element refers to a code block which can be found in
			//the <available_CControls> section, to be substitued in for every
			//object.
			else if(child->Type() == 1) //for some reason, ELEMENT is picked up as an undeclared identifier?
			{
				WriteCodeBlock(&source, &doc, child->Value());
			}
			else //everything else is text, write it to the file
			{
				temptxt = child->ToText();
				if(temptxt)
				{
					tempstr = temptxt->Value();
					source.Write(tempstr);
				}
			}
		}
	}
}

//------------------------
void VSTMainGUIPanel::LoadBackgroundImage(wxString path)
{
	if(!Background->LoadFile(path))
	{
		wxMessageDialog tempdlg(this, wxString("Could not load the background image for some reason"), wxString("Not the face! Not the face!"));
		tempdlg.ShowModal();
		ImageIsLoaded = false;
	}
	else
		ImageIsLoaded = true;

	Refresh();
}

//------------------------
void VSTMainGUIPanel::InsertControl()
{
	GenericCControl *tempcontrol_type;
	GenericCControl *tempcontrol;
	wxImage *tempimage;
	char *tempch;
	long image_width, image_height;

	InsertControlDialog dlg(this, -1, wxString("Insert CControl"), &AvailableControls);

	if(dlg.ShowModal() == wxID_OK)
	{
		tempcontrol_type = dlg.GetControl();
		tempcontrol = new GenericCControl(*tempcontrol_type);

		ControlsInUse.Add(tempcontrol);

		if(tempcontrol->GetHasGUIImage())
		{
			wxFileDialog dlg2(this, wxString("Choose an image file"), wxString(""), wxString(""), wxString("png files (*.png)|*.png|bmp files (*.bmp)|*.bmp"), wxOPEN);

			if(dlg2.ShowModal() == wxID_OK)
			{
				tempimage = new wxImage(dlg2.GetPath());

				//the first image is always the one displayed by VSTGUI Builder
				tempch = GetFirstImage(tempcontrol_type->GetCControlName());
				tempcontrol->SetImage(tempimage, tempch);
				tempcontrol->SetImagePath(dlg2.GetPath(), tempch);
				delete [] tempch;

				image_width = tempimage->GetWidth();
				image_height = tempimage->GetHeight();
				image_height /= tempcontrol->GetNumStacked();

				tempcontrol->SetSize(image_width, image_height);

				Refresh();
			}
		}
		else
		{
			tempcontrol->SetSize(20, 20);
			Refresh();
		}
	}
}

//------------------------
char *VSTMainGUIPanel::GetFirstImage(char *controltype)
{
	wxString tempstr;
	char *tempch;

	TiXmlDocument doc("DefaultControls.xml");
	if(!doc.LoadFile())
	{
		tempstr.Printf("Could not load config file 'DefaultControls.xml'. Error='%s'.\n", doc.ErrorDesc());
		wxMessageDialog tempdlg(this, tempstr, wxString("Please!  Not the glass figurine!"));
		tempdlg.ShowModal();
	}

	TiXmlHandle docHandle(&doc);

	TiXmlElement *firstimage = docHandle.FirstChild("root").FirstChild("available_CControls").FirstChild(controltype).FirstChild("images").FirstChildElement().Element();
	if(firstimage)
	{
		tempch = new char[128];
		strcpy(tempch, firstimage->Value());
		return tempch;
	}
	else
		return 0;
}

//------------------------
void VSTMainGUIPanel::WriteImageHandles()
{
	unsigned long i, j, k;
	wxArrayString temparr;
	char tempstr[128];
	wxString tempstr2, tempstr3;
	int tempint;
	bool already_here = false;

	ImageHandles.Empty();


	for(i=0;i<ControlsInUse.GetCount();i++)
	{
		for(j=0;j<ControlsInUse.Item(i).GetNumImages();j++)
		{
			strcpy(tempstr, ControlsInUse.Item(i).GetImageName(j));
			if(strcmp(ControlsInUse.Item(i).GetImagePath(tempstr), "") != 0)
			{
				already_here = false;
				for(k=0;k<temparr.GetCount();k++)
				{
					if(!temparr.Item(k).Cmp(ControlsInUse.Item(i).GetImagePath(tempstr)))
					{
						already_here = true;
						break;
					}
				}
				if(!already_here)
				{
					if(ControlsInUse.Item(i).GetImagePath(tempstr).Cmp(wxString("hBack")) != 0)
						temparr.Add(ControlsInUse.Item(i).GetImagePath(tempstr));
				}
			}
		}
	}

	for(i=0;i<temparr.GetCount();i++)
	{
		tempstr3 = "h";
		tempstr2 = temparr.Item(i);
		tempint = tempstr2.Find('\\', TRUE); //find last slash in path
		tempint++;
		for(j=tempint;j<(tempstr2.Length()-4);j++)//skip .png/.bmp
			tempstr3 += tempstr2.GetChar(j); //handle = h<filename-extension>
		ImageHandles.Add(tempstr3);

		for(j=0;j<ControlsInUse.GetCount();j++)
		{
			for(k=0;k<ControlsInUse.Item(j).GetNumImages();k++)
			{
				strcpy(tempstr, ControlsInUse.Item(j).GetImageName(k));
				if(strcmp(ControlsInUse.Item(j).GetImagePath(tempstr), "") != 0)
				{
					if(!strcmp(ControlsInUse.Item(j).GetImagePath(tempstr), temparr.Item(i)))
					{
						ControlsInUse.Item(j).SetImageHandle(tempstr3, tempstr);
					}
				}
			}			
		}
	}
}

//------------------------
void VSTMainGUIPanel::WriteCodeBlock(wxFile *file, TiXmlDocument *doc, const char *name)
{
	unsigned long i, j;
	TiXmlHandle docHandle(doc);
	TiXmlNode *parent = 0;
	TiXmlNode *child = 0;
	TiXmlText *temptext = 0;
	wxString tempstr;

	for(i=0;i<ControlsInUse.GetCount();i++)
	{
		parent = docHandle.FirstChild("root").FirstChild("available_CControls").FirstChild(ControlsInUse.Item(i).GetCControlName()).FirstChild("code").FirstChild(name).Node();
		while(child = parent->IterateChildren(child))
		{
			/*if(!strcmp(child->Value(), "name")) //replace <name/> with control's name
			{
				file->Write(ControlsInUse.Item(i).GetProperty("name"));
			}
			else */if(!strcmp(child->Value(), "br")) //replace <br/> with a line break
			{
				file->Write(wxString("\n"));
			}
			else if(!strcmp(child->Value(), "nbsp")) //replace <nbsp/> with a space
			{
				file->Write(wxString(" "));
			}
			else if(!strcmp(child->Value(), "tab")) //replace <tab/> with a tab
			{
				file->Write(wxString("    ")); //tab = 4 spaces
			}
			else if(child->Type() == 1) //for some reason, ELEMENT is picked up as an undeclared identifier?
			{
				file->Write(ControlsInUse.Item(i).GetProperty(child->Value()));
				if(ControlsInUse.Item(i).GetImageHandle(child->Value()))
					file->Write(ControlsInUse.Item(i).GetImageHandle(child->Value()));
			}
			else
			{
				temptext = child->ToText();
				if(temptext)
				{
					tempstr = temptext->Value();
					file->Write(tempstr);
				}
			}
		}
	}
}